
#include "TArtSimpleFunction.hh"
#include "TArtHistManager.hh"
#include "TArtCanvasManager.hh"
#include "TArtGlobal.hh"
#include "TArtExec.hh"

#include "TH1.h"
#include "TH1D.h"
#include "TH2.h"
#include "TH3.h"
#include "TProfile.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TDirectory.h"
#include "TList.h"
#include "TCutG.h"
#include "TPad.h"
#include "TCanvas.h"

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <algorithm>

TH2* TArtSimpleFunction::CutGHist()
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);

  if(!gPad) return 0;
  TCutG* cutg = static_cast<TCutG*>(gPad->WaitPrimitive("CUTG","CutG"));
  if(!cutg) return 0;
  cutg->SetName("cutg_cpro");

  std::string name(hist2->GetName());
  name += "_cpro";
  name = TArtGlobal::IncrementName(name);
  TObject* obj = hist2->Clone(name.c_str());
  if(!obj) return 0;
  TH2* hist2cpro = static_cast<TH2*>(obj);
  hist2cpro->Reset();

  //0:under flow, nbin+1:over flow
  Double_t entries = 0;
  for(Int_t biny=0; biny<hist2->GetNbinsY()+2; ++biny){
    for(Int_t binx=0; binx<hist2->GetNbinsX()+2; ++binx){
      Double_t x = hist2->GetXaxis()->GetBinCenter(binx);
      Double_t y = hist2->GetYaxis()->GetBinCenter(biny);
      if(cutg->IsInside(x,y)){
	Double_t bincontent = hist2->GetBinContent(binx, biny);
	hist2cpro->SetBinContent(binx, biny, bincontent);
	entries += bincontent;
      }
    }
  }
  hist2cpro->SetEntries(entries);

  if(!TArtHistManager::InsertAfterHist(hist2, hist2cpro)){
    hist2cpro->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(hist2cpro);

  return hist2cpro;
}

TFitResultPtr TArtSimpleFunction::Fit(const char* formula, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  TH1* hist = gTH1;
  if(hist){
    return hist->Fit(formula, option, goption, xmin, xmax);
  }else{
   return TFitResultPtr();
  }
}

TFitResultPtr TArtSimpleFunction::FitPolynomial(int n, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  std::ostringstream os;
  os << n;
  std::string pol("pol" + os.str());

  return Fit(pol.c_str(), option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitExponential(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("expo", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitGaussian(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("gaus", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitLandau(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("landau", option, goption, xmin, xmax);
}

TProfile* TArtSimpleFunction::ProfileX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_pfx";
  name = TArtGlobal::IncrementName(name);
  TProfile* prof = hist2->ProfileX(name.c_str(), firstybin, lastybin, option);
  prof->SetMinimum(hist2->GetYaxis()->GetXmin());
  prof->SetMaximum(hist2->GetYaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  if(!TArtHistManager::InsertAfterHist(hist2, prof)){
    prof->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TProfile* TArtSimpleFunction::ProfileY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_pfy";
  name = TArtGlobal::IncrementName(name);
  TProfile* prof = hist2->ProfileY(name.c_str(), firstxbin, lastxbin, option);
  prof->SetMinimum(hist2->GetXaxis()->GetXmin());
  prof->SetMaximum(hist2->GetXaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  if(!TArtHistManager::InsertAfterHist(hist2, prof)){
    prof->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TH1D* TArtSimpleFunction::ProjectionX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_px";
  name = TArtGlobal::IncrementName(name);
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    histd->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::ProjectionY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_py";
  name = TArtGlobal::IncrementName(name);
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    histd->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::BanY(double firsty, double lasty, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_bny";
  name = TArtGlobal::IncrementName(name);
  Int_t firstybin = hist2->GetYaxis()->FindBin(firsty);
  Int_t lastybin  = hist2->GetYaxis()->FindBin(lasty);
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    histd->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::BanX(double firstx, double lastx, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_bnx";
  name = TArtGlobal::IncrementName(name);
  Int_t firstxbin = hist2->GetXaxis()->FindBin(firstx);
  Int_t lastxbin  = hist2->GetXaxis()->FindBin(lastx);
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    histd->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1* TArtSimpleFunction::Range(double firstx, double lastx, double firsty, double lasty, double firstz, double lastz)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  if(hist->GetDimension() == 1){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
  }else if(hist->GetDimension() == 2){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
    hist->GetYaxis()->SetRangeUser(firsty, lasty);
  }else if(hist->GetDimension() == 3){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
    hist->GetYaxis()->SetRangeUser(firsty, lasty);
    hist->GetZaxis()->SetRangeUser(firstz, lastz);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeX(double firstx, double lastx)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  hist->GetXaxis()->SetRangeUser(firstx, lastx);

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeY(double firsty, double lasty)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() == 1) return 0;
  
  hist->GetYaxis()->SetRangeUser(firsty, lasty);

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeZ(double firstz, double lastz)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 3) return 0;

  hist->GetZaxis()->SetRangeUser(firstz, lastz);

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::Blow(double firstx, double lastx, double firsty, double lasty, double firstz, double lastz)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  if(firstx>lastx){
    firstx = gTH1->GetXaxis()->GetXmin();
    lastx = gTH1->GetXaxis()->GetXmax();
  }
  if(firsty>lasty){
    firsty = gTH1->GetYaxis()->GetXmin();
    lasty = gTH1->GetYaxis()->GetXmax();
  }
  if(firstz>lastz){
    firstz = gTH1->GetZaxis()->GetXmin();
    lastz = gTH1->GetZaxis()->GetXmax();
  }

  std::string name(hist->GetName());
  name += "_blow";
  name = TArtGlobal::IncrementName(name);
  TObject* obj = hist->Clone(name.c_str());
  if(!obj) return 0;
  TH1* histblow = static_cast<TH1*>(obj);
  histblow->Reset();
  Double_t entries = 0;
  Int_t dimension = histblow->GetDimension();
  if(dimension == 1){
    for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
      Double_t x = hist->GetXaxis()->GetBinCenter(binx);
      if(x >= firstx && x <= lastx){
	Double_t bincontent = hist->GetBinContent(binx);
	histblow->SetBinContent(binx, bincontent);
	entries += bincontent;
      }
    }
  }else if(dimension == 2){
    for(Int_t biny=0; biny<hist->GetNbinsY()+2; ++biny){
      for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
	Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	if((x >= firstx && x <= lastx) &&
	   (y >= firsty && y <= lasty)){
	  Double_t bincontent = hist->GetBinContent(binx, biny);
	  histblow->SetBinContent(binx, biny, bincontent);
	  entries += bincontent;
	}
      }
    }
  }else if(dimension == 3){
    for(Int_t binz=0; binz<hist->GetNbinsZ()+2; ++binz){
      for(Int_t biny=0; biny<hist->GetNbinsY()+2; ++biny){
	for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
	  Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	  Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	  Double_t z = hist->GetZaxis()->GetBinCenter(binz);
	  if((x >= firstx && x <= lastx) &&
	     (y >= firsty && y <= lasty) &&
	     (z >= firstz && z <= lastz)){
	    Double_t bincontent = hist->GetBinContent(binx, biny, binz);
	    histblow->SetBinContent(binx, biny, binz, bincontent);
	    entries += bincontent;
	  }
	}
      }
    }
  }
  histblow->SetEntries(entries);

  if(!TArtHistManager::InsertAfterHist(hist, histblow)){
    histblow->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histblow);

  return histblow;
}

TH1* TArtSimpleFunction::BlowX(double firstx, double lastx)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  return Blow(firstx, lastx);
}

TH1* TArtSimpleFunction::BlowY(double firsty, double lasty)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  return Blow(gTH1->GetXaxis()->GetXmin(), gTH1->GetXaxis()->GetXmax(),
	      firsty, lasty);
}

TH1* TArtSimpleFunction::BlowZ(double firstz, double lastz)
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  return Blow(gTH1->GetXaxis()->GetXmin(), gTH1->GetXaxis()->GetXmax(),
	      gTH1->GetYaxis()->GetXmin(), gTH1->GetYaxis()->GetXmax(),
	      firstz, lastz);
}

TH1* TArtSimpleFunction::GBlowX()
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t x1 = TArtExec::GXValue();
  Double_t x2 = TArtExec::GXValue();
  if(x1>x2) std::swap(x1,x2);

  return Blow(x1,x2);
}

TH1* TArtSimpleFunction::GBlowY()
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t y1 = TArtExec::GYValue();
  Double_t y2 = TArtExec::GYValue();
  if(y1>y2) std::swap(y1,y2);

  return Blow(y1,y2);
}

TH1* TArtSimpleFunction::GBlowXY()
{
  TH1* hist = gTH1;
  if(!hist) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t x1,y1;
  Double_t x2,y2;
  TArtExec::GXYValue(x1,y1);
  TArtExec::GXYValue(x2,y2);
  if(x1>x2) std::swap(x1,x2);
  if(y1>y2) std::swap(y1,y2);

  return Blow(x1,x2,y1,y2);
}

TH1* TArtSimpleFunction::MinMax(double min, double max)
{
  if(min >= max) return 0;

  TH1* hist = gTH1;
  if(!hist) return 0;

  std::string name = std::string(hist->GetName()) + "_mami";
  name = TArtGlobal::IncrementName(name);  
  TObject* obj = hist->Clone(name.c_str());
  if(!obj) return 0;
  TH1* histmami = static_cast<TH1*>(obj);
  histmami->SetMinimum(min);
  histmami->SetMaximum(max);

  if(!TArtHistManager::InsertAfterHist(hist, histmami)){
    histmami->Delete();
    return 0;
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histmami);

  return histmami;
}
